@@ -36,7 +36,9 @@
 #include "gdata-gd-phone-number.h"
 #include "gdata-parsable.h"
 #include "gdata-parser.h"
+#include "gdata-comparable.h"
 
+static void gdata_gd_phone_number_comparable_init (GDataComparableIface *iface);
 static void gdata_gd_phone_number_finalize (GObject *object);
 static void gdata_gd_phone_number_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
 static void gdata_gd_phone_number_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec);
@@ -62,8 +64,8 @@ enum {
 	PROP_IS_PRIMARY
 };
 
-G_DEFINE_TYPE (GDataGDPhoneNumber, gdata_gd_phone_number, GDATA_TYPE_PARSABLE)
-#define GDATA_GD_PHONE_NUMBER_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GDATA_TYPE_GD_PHONE_NUMBER, GDataGDPhoneNumberPrivate))
+G_DEFINE_TYPE_WITH_CODE (GDataGDPhoneNumber, gdata_gd_phone_number, GDATA_TYPE_PARSABLE,
+                         G_IMPLEMENT_INTERFACE (GDATA_TYPE_COMPARABLE, gdata_gd_phone_number_comparable_init))
 
 static void
 gdata_gd_phone_number_class_init (GDataGDPhoneNumberClass *klass)
@@ -96,15 +98,15 @@ gdata_gd_phone_number_class_init (GDataGDPhoneNumberClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_NUMBER,
-				g_param_spec_string ("number",
-					"Number", "Human-readable phone number; may be in any telephone number format.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("number",
+	                                                      "Number", "Human-readable phone number; may be in any telephone number format.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataGDPhoneNumber:uri:
 	 *
-	 * An optional "tel URI" used to represent the number in a formal way, useful for programmatic access, such as a VoIP/PSTN bridge.
+	 * An optional "tel URI" used to represent the number in a formal way. Useful for programmatic access, such as a VoIP/PSTN bridge.
 	 *
 	 * For more information, see the
 	 * <ulink type="http" url="http://code.google.com/apis/gdata/docs/2.0/elements.html#gdPhoneNumber">GData specification</ulink>.
@@ -112,15 +114,15 @@ gdata_gd_phone_number_class_init (GDataGDPhoneNumberClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_URI,
-				g_param_spec_string ("uri",
-					"URI", "An optional \"tel URI\" used to represent the number in a formal way, useful for programmatic access.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("uri",
+	                                                      "URI", "An optional \"tel URI\" used to represent the number in a formal way.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataGDPhoneNumber:relation-type:
 	 *
-	 * A programmatic value that identifies the type of phone number.
+	 * A programmatic value that identifies the type of phone number. For example: %GDATA_GD_PHONE_NUMBER_WORK or %GDATA_GD_PHONE_NUMBER_PAGER.
 	 *
 	 * For more information, see the
 	 * <ulink type="http" url="http://code.google.com/apis/gdata/docs/2.0/elements.html#gdPhoneNumber">GData specification</ulink>.
@@ -128,10 +130,10 @@ gdata_gd_phone_number_class_init (GDataGDPhoneNumberClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_RELATION_TYPE,
-				g_param_spec_string ("relation-type",
-					"Relation type", "A programmatic value that identifies the type of phone number.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("relation-type",
+	                                                      "Relation type", "A programmatic value that identifies the type of phone number.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataGDPhoneNumber:label:
@@ -144,10 +146,10 @@ gdata_gd_phone_number_class_init (GDataGDPhoneNumberClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_LABEL,
-				g_param_spec_string ("label",
-					"Label", "A simple string value used to name this phone number.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("label",
+	                                                      "Label", "A simple string value used to name this phone number.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataGDPhoneNumber:is-primary:
@@ -160,10 +162,22 @@ gdata_gd_phone_number_class_init (GDataGDPhoneNumberClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_IS_PRIMARY,
-				g_param_spec_boolean ("is-primary",
-					"Primary?", "Indicates which phone number out of a group is primary.",
-					FALSE,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_boolean ("is-primary",
+	                                                       "Primary?", "Indicates which phone number out of a group is primary.",
+	                                                       FALSE,
+	                                                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+}
+
+static gint
+compare_with (GDataComparable *self, GDataComparable *other)
+{
+	return g_strcmp0 (((GDataGDPhoneNumber*) self)->priv->number, ((GDataGDPhoneNumber*) other)->priv->number);
+}
+
+static void
+gdata_gd_phone_number_comparable_init (GDataComparableIface *iface)
+{
+	iface->compare_with = compare_with;
 }
 
 static void
@@ -245,10 +259,14 @@ gdata_gd_phone_number_set_property (GObject *object, guint property_id, const GV
 static gboolean
 pre_parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *root_node, gpointer user_data, GError **error)
 {
-	xmlChar *number, *rel, *primary;
+	xmlChar *number, *rel;
 	gboolean primary_bool;
 	GDataGDPhoneNumberPrivate *priv = GDATA_GD_PHONE_NUMBER (parsable)->priv;
 
+	/* Is it the primary phone number? */
+	if (gdata_parser_boolean_from_property (root_node, "primary", &primary_bool, 0, error) == FALSE)
+		return FALSE;
+
 	number = xmlNodeListGetString (doc, root_node->children, TRUE);
 	if (number == NULL || *number == '\0') {
 		xmlFree (number);
@@ -262,21 +280,6 @@ pre_parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *root_node, gpointe
 		return gdata_parser_error_required_property_missing (root_node, "rel", error);
 	}
 
-	/* Is it the primary phone number? */
-	primary = xmlGetProp (root_node, (xmlChar*) "primary");
-	if (primary == NULL || xmlStrcmp (primary, (xmlChar*) "false") == 0)
-		primary_bool = FALSE;
-	else if (xmlStrcmp (primary, (xmlChar*) "true") == 0)
-		primary_bool = TRUE;
-	else {
-		gdata_parser_error_unknown_property_value (root_node, "primary", (gchar*) primary, error);
-		xmlFree (primary);
-		xmlFree (rel);
-		xmlFree (number);
-		return FALSE;
-	}
-	xmlFree (primary);
-
 	gdata_gd_phone_number_set_number (GDATA_GD_PHONE_NUMBER (parsable), (gchar*) number);
 	priv->uri = (gchar*) xmlGetProp (root_node, (xmlChar*) "uri");
 	priv->relation_type = (gchar*) rel;
@@ -295,12 +298,7 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 	if (node->type != XML_ELEMENT_NODE)
 		return TRUE;
 
-	if (GDATA_PARSABLE_CLASS (gdata_gd_phone_number_parent_class)->parse_xml (parsable, doc, node, user_data, error) == FALSE) {
-		/* Error! */
-		return FALSE;
-	}
-
-	return TRUE;
+	return GDATA_PARSABLE_CLASS (gdata_gd_phone_number_parent_class)->parse_xml (parsable, doc, node, user_data, error);
 }
 
 static void
@@ -352,38 +350,10 @@ get_namespaces (GDataParsable *parsable, GHashTable *namespaces)
 GDataGDPhoneNumber *
 gdata_gd_phone_number_new (const gchar *number, const gchar *relation_type, const gchar *label, const gchar *uri, gboolean is_primary)
 {
+	g_return_val_if_fail (number != NULL && *number != '\0', NULL);
 	g_return_val_if_fail (relation_type == NULL || *relation_type != '\0', NULL);
 	return g_object_new (GDATA_TYPE_GD_PHONE_NUMBER, "number", number, "uri", uri, "relation-type", relation_type,
-			     "label", label, "is-primary", is_primary, NULL);
-}
-
-/**
- * gdata_gd_phone_number_compare:
- * @a: a #GDataGDPhoneNumber, or %NULL
- * @b: another #GDataGDPhoneNumber, or %NULL
- *
- * Compares the two phone numbers in a strcmp() fashion. %NULL values are handled gracefully, with
- * <code class="literal">0</code> returned if both @a and @b are %NULL, <code class="literal">-1</code> if @a is %NULL
- * and <code class="literal">1</code> if @b is %NULL.
- *
- * The comparison of non-%NULL values is done on the basis of the @number property of the #GDataGDPhoneNumber<!-- -->s.
- *
- * Return value: <code class="literal">0</code> if @a equals @b, <code class="literal">-1</code> or <code class="literal">1</code> as
- * appropriate otherwise
- *
- * Since: 0.4.0
- **/
-gint
-gdata_gd_phone_number_compare (const GDataGDPhoneNumber *a, const GDataGDPhoneNumber *b)
-{
-	if (a == NULL && b != NULL)
-		return -1;
-	else if (a != NULL && b == NULL)
-		return 1;
-
-	if (a == b)
-		return 0;
-	return g_strcmp0 (a->priv->number, b->priv->number);
+	                     "label", label, "is-primary", is_primary, NULL);
 }
 
 /**
