@@ -37,38 +37,49 @@ G_BEGIN_DECLS
 
 typedef struct _GDataCalendarQueryPrivate	GDataCalendarQueryPrivate;
 
+/**
+ * GDataCalendarQuery:
+ *
+ * All the fields in the #GDataCalendarQuery structure are private and should never be accessed directly.
+ **/
 typedef struct {
 	GDataQuery parent;
 	GDataCalendarQueryPrivate *priv;
 } GDataCalendarQuery;
 
+/**
+ * GDataCalendarQueryClass:
+ *
+ * All the fields in the #GDataCalendarQueryClass structure are private and should never be accessed directly.
+ **/
 typedef struct {
+	/*< private >*/
 	GDataQueryClass parent;
 } GDataCalendarQueryClass;
 
-GType gdata_calendar_query_get_type (void);
+GType gdata_calendar_query_get_type (void) G_GNUC_CONST;
 
-GDataCalendarQuery *gdata_calendar_query_new (const gchar *q);
-GDataCalendarQuery *gdata_calendar_query_new_with_limits (const gchar *q, GTimeVal *start_min, GTimeVal *start_max);
+GDataCalendarQuery *gdata_calendar_query_new (const gchar *q) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;
+GDataCalendarQuery *gdata_calendar_query_new_with_limits (const gchar *q, gint64 start_min, gint64 start_max) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;
 
-gboolean gdata_calendar_query_get_future_events (GDataCalendarQuery *self);
+gboolean gdata_calendar_query_get_future_events (GDataCalendarQuery *self) G_GNUC_PURE;
 void gdata_calendar_query_set_future_events (GDataCalendarQuery *self, gboolean future_events);
-const gchar *gdata_calendar_query_get_order_by (GDataCalendarQuery *self);
+const gchar *gdata_calendar_query_get_order_by (GDataCalendarQuery *self) G_GNUC_PURE;
 void gdata_calendar_query_set_order_by (GDataCalendarQuery *self, const gchar *order_by);
-void gdata_calendar_query_get_recurrence_expansion_start (GDataCalendarQuery *self, GTimeVal *start);
-void gdata_calendar_query_set_recurrence_expansion_start (GDataCalendarQuery *self, GTimeVal *start);
-void gdata_calendar_query_get_recurrence_expansion_end (GDataCalendarQuery *self, GTimeVal *end);
-void gdata_calendar_query_set_recurrence_expansion_end (GDataCalendarQuery *self, GTimeVal *end);
-gboolean gdata_calendar_query_get_single_events (GDataCalendarQuery *self);
+gint64 gdata_calendar_query_get_recurrence_expansion_start (GDataCalendarQuery *self);
+void gdata_calendar_query_set_recurrence_expansion_start (GDataCalendarQuery *self, gint64 start);
+gint64 gdata_calendar_query_get_recurrence_expansion_end (GDataCalendarQuery *self);
+void gdata_calendar_query_set_recurrence_expansion_end (GDataCalendarQuery *self, gint64 end);
+gboolean gdata_calendar_query_get_single_events (GDataCalendarQuery *self) G_GNUC_PURE;
 void gdata_calendar_query_set_single_events (GDataCalendarQuery *self, gboolean single_events);
-const gchar *gdata_calendar_query_get_sort_order (GDataCalendarQuery *self);
+const gchar *gdata_calendar_query_get_sort_order (GDataCalendarQuery *self) G_GNUC_PURE;
 void gdata_calendar_query_set_sort_order (GDataCalendarQuery *self, const gchar *sort_order);
-void gdata_calendar_query_get_start_min (GDataCalendarQuery *self, GTimeVal *start_min);
-void gdata_calendar_query_set_start_min (GDataCalendarQuery *self, GTimeVal *start_min);
-void gdata_calendar_query_get_start_max (GDataCalendarQuery *self, GTimeVal *start_max);
-void gdata_calendar_query_set_start_max (GDataCalendarQuery *self, GTimeVal *start_max);
-const gchar *gdata_calendar_query_get_ctz (GDataCalendarQuery *self);
-void gdata_calendar_query_set_ctz (GDataCalendarQuery *self, const gchar *ctz);
+gint64 gdata_calendar_query_get_start_min (GDataCalendarQuery *self);
+void gdata_calendar_query_set_start_min (GDataCalendarQuery *self, gint64 start_min);
+gint64 gdata_calendar_query_get_start_max (GDataCalendarQuery *self);
+void gdata_calendar_query_set_start_max (GDataCalendarQuery *self, gint64 start_max);
+const gchar *gdata_calendar_query_get_timezone (GDataCalendarQuery *self) G_GNUC_PURE;
+void gdata_calendar_query_set_timezone (GDataCalendarQuery *self, const gchar *_timezone);
 
 G_END_DECLS
 
