@@ -29,6 +29,119 @@
  * For more details of Google Contacts' GData API, see the <ulink type="http" url="http://code.google.com/apis/contacts/docs/2.0/reference.html">
  * online documentation</ulink>.
  *
+ * <example>
+ * 	<title>Querying for Groups</title>
+ * 	<programlisting>
+ *	GDataContactsService *service;
+ *	GDataFeed *feed;
+ *	GList *i;
+ *	GError *error = NULL;
+ *
+ *	/<!-- -->* Create a service *<!-- -->/
+ *	service = create_contacts_service ();
+ *
+ *	/<!-- -->* Query for groups *<!-- -->/
+ *	feed = gdata_contacts_service_query_groups (service, NULL, NULL, NULL, NULL, &error);
+ *
+ *	g_object_unref (service);
+ *
+ *	if (error != NULL) {
+ *		g_error ("Error querying for groups: %s", error->message);
+ *		g_error_free (error);
+ *		return;
+ *	}
+ *
+ *	/<!-- -->* Iterate through the returned groups and do something with them *<!-- -->/
+ *	for (i = gdata_feed_get_entries (feed); i != NULL; i = i->next) {
+ *		const gchar *system_group_id, *group_name;
+ *		gboolean is_system_group;
+ *		GDataContactsGroup *group = GDATA_CONTACTS_GROUP (i->data);
+ *
+ *		/<!-- -->* Determine whether the group's a system group. If so, you should use the system group ID to provide your application's own
+ *		 * translations of the group name, as it's not translated. *<!-- -->/
+ *		system_group_id = gdata_contacts_group_get_system_group_id (group);
+ *		is_system_group = (system_group_id != NULL) ? TRUE : FALSE;
+ *		group_name = (is_system_group == TRUE) ? get_group_name_for_system_group_id (system_group_id)
+ *		                                       : gdata_entry_get_title (GDATA_ENTRY (group));
+ *
+ *		/<!-- -->* Do something with the group here, such as insert it into a UI. Note that system groups are not allowed to be deleted,
+ *		 * so you may want to make certain parts of your UI insensitive accordingly if the group is a system group. *<!-- -->/
+ *	}
+ *
+ *	g_object_unref (feed);
+ * 	</programlisting>
+ * </example>
+ *
+ * The Contacts service can be manipulated using batch operations, too. See the
+ * <ulink type="http" url="http://code.google.com/apis/contacts/docs/3.0/developers_guide_protocol.html#Batch">online documentation on batch
+ * operations</ulink> for more information.
+ *
+ * <example>
+ * 	<title>Performing a Batch Operation on Contacts</title>
+ * 	<programlisting>
+ *	GDataContactsService *service;
+ *	GDataBatchOperation *operation;
+ *	GDataFeed *feed;
+ *	GDataLink *batch_link;
+ *	GList *i;
+ *	GError *error = NULL;
+ *
+ *	/<!-- -->* Create a service *<!-- -->/
+ *	service = create_contacts_service ();
+ *
+ *	/<!-- -->* Create the batch operation; this requires that we have done a query first so that we can get the batch link *<!-- -->/
+ *	feed = do_some_query (service);
+ *	batch_link = gdata_feed_look_up_link (feed, GDATA_LINK_BATCH);
+ *	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), gdata_link_get_uri (batch_link));
+ *	g_object_unref (feed);
+ *
+ *	gdata_batch_operation_add_query (operation, contact_entry_id_to_query, GDATA_TYPE_CONTACTS_CONTACT,
+ *	                                 (GDataBatchOperationCallback) batch_query_cb, user_data);
+ *	gdata_batch_operation_add_insertion (operation, new_entry, (GDataBatchOperationCallback) batch_insertion_cb, user_data);
+ *	gdata_batch_operation_add_update (operation, old_entry, (GDataBatchOperationCallback) batch_update_cb, user_data);
+ *	gdata_batch_operation_add_deletion (operation, entry_to_delete, (GDataBatchOperationCallback) batch_deletion_cb, user_data);
+ *
+ *	/<!-- -->* Run the batch operation and handle the results in the various callbacks *<!-- -->/
+ *	gdata_test_batch_operation_run (operation, NULL, &error);
+ *
+ *	g_object_unref (operation);
+ *	g_object_unref (service);
+ *
+ *	if (error != NULL) {
+ *		g_error ("Error running batch operation: %s", error->message);
+ *		g_error_free (error);
+ *		return;
+ *	}
+ *
+ *	static void
+ *	batch_query_cb (guint operation_id, GDataBatchOperationType operation_type, GDataEntry *entry, GError *error, gpointer user_data)
+ *	{
+ *		/<!-- -->* operation_type == GDATA_BATCH_OPERATION_QUERY *<!-- -->/
+ *		/<!-- -->* Reference and do something with the returned entry. *<!-- -->/
+ *	}
+ *
+ *	static void
+ *	batch_insertion_cb (guint operation_id, GDataBatchOperationType operation_type, GDataEntry *entry, GError *error, gpointer user_data)
+ *	{
+ *		/<!-- -->* operation_type == GDATA_BATCH_OPERATION_INSERTION *<!-- -->/
+ *		/<!-- -->* Reference and do something with the returned entry. *<!-- -->/
+ *	}
+ *
+ *	static void
+ *	batch_update_cb (guint operation_id, GDataBatchOperationType operation_type, GDataEntry *entry, GError *error, gpointer user_data)
+ *	{
+ *		/<!-- -->* operation_type == GDATA_BATCH_OPERATION_UPDATE *<!-- -->/
+ *		/<!-- -->* Reference and do something with the returned entry. *<!-- -->/
+ *	}
+ *
+ *	static void
+ *	batch_deletion_cb (guint operation_id, GDataBatchOperationType operation_type, GDataEntry *entry, GError *error, gpointer user_data)
+ *	{
+ *		/<!-- -->* operation_type == GDATA_BATCH_OPERATION_DELETION, entry == NULL *<!-- -->/
+ *	}
+ * 	</programlisting>
+ * </example>
+ *
  * Since: 0.2.0
  **/
 
