@@ -226,6 +226,40 @@ test_query_events (gconstpointer service)
 	g_object_unref (calendar);
 }
 
+static void
+test_query_events_async_cb (GDataService *service, GAsyncResult *async_result, GMainLoop *main_loop)
+{
+	GDataFeed *feed;
+	GError *error = NULL;
+
+	feed = gdata_service_query_finish (service, async_result, &error);
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_CALENDAR_FEED (feed));
+
+	/* TODO: Tests? */
+	g_main_loop_quit (main_loop);
+
+	g_object_unref (feed);
+}
+
+static void
+test_query_events_async (gconstpointer service)
+{
+	GDataCalendarCalendar *calendar;
+	GMainLoop *main_loop;
+	GError *error = NULL;
+
+	calendar = get_calendar (service, &error);
+	main_loop = g_main_loop_new (NULL, TRUE);
+
+	gdata_calendar_service_query_events_async (GDATA_CALENDAR_SERVICE (service), calendar, NULL, NULL, NULL, NULL,
+	                                           (GAsyncReadyCallback) test_query_events_async_cb, main_loop);
+	g_main_loop_run (main_loop);
+
+	g_main_loop_unref (main_loop);
+	g_object_unref (calendar);
+}
+
 static void
 test_insert_simple (gconstpointer service)
 {
@@ -234,7 +268,6 @@ test_insert_simple (gconstpointer service)
 	GDataGDWho *who;
 	GDataGDWhen *when;
 	GTimeVal start_time, end_time;
-	gchar *xml;
 	GError *error = NULL;
 
 	event = gdata_calendar_event_new (NULL);
@@ -256,8 +289,7 @@ test_insert_simple (gconstpointer service)
 	g_object_unref (when);
 
 	/* Check the XML */
-	xml = gdata_parsable_get_xml (GDATA_PARSABLE (event));
-	g_assert_cmpstr (xml, ==,
+	gdata_test_assert_xml (event,
 			 "<?xml version='1.0' encoding='UTF-8'?>"
 			 "<entry xmlns='http://www.w3.org/2005/Atom' "
 			 	"xmlns:gd='http://schemas.google.com/g/2005' "
@@ -278,7 +310,6 @@ test_insert_simple (gconstpointer service)
 			 		"valueString='John Smith\342\200\275'/>"
 			 	"<gd:where valueString='Rolling Lawn Courts'/>"
 			 "</entry>");
-	g_free (xml);
 
 	/* Insert the event */
 	new_event = gdata_calendar_service_insert_event (GDATA_CALENDAR_SERVICE (service), event, NULL, &error);
@@ -292,6 +323,63 @@ test_insert_simple (gconstpointer service)
 	g_object_unref (new_event);
 }
 
+static void
+test_insert_simple_async_cb (GDataService *service, GAsyncResult *result, GMainLoop *main_loop)
+{
+	GDataEntry *event;
+	GError *error = NULL;
+
+	event = gdata_service_insert_entry_finish (service, result, &error);
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_CALENDAR_EVENT (event));
+
+	/* Check the event is correct */
+	g_assert_cmpstr (gdata_entry_get_title (event), ==, "Tennis with Beth");
+
+	g_main_loop_quit (main_loop);
+
+	g_object_unref (event);
+}
+
+static void
+test_insert_simple_async (gconstpointer service)
+{
+	GDataCalendarEvent *event;
+	GDataGDWhere *where;
+	GDataGDWho *who;
+	GDataGDWhen *when;
+	GTimeVal start_time, end_time;
+	GMainLoop *main_loop;
+
+	event = gdata_calendar_event_new (NULL);
+
+	gdata_entry_set_title (GDATA_ENTRY (event), "Tennis with Beth");
+	gdata_entry_set_content (GDATA_ENTRY (event), "Meet for a quick lesson.");
+	gdata_calendar_event_set_transparency (event, GDATA_GD_EVENT_TRANSPARENCY_OPAQUE);
+	gdata_calendar_event_set_status (event, GDATA_GD_EVENT_STATUS_CONFIRMED);
+	where = gdata_gd_where_new (NULL, "Rolling Lawn Courts", NULL);
+	gdata_calendar_event_add_place (event, where);
+	g_object_unref (where);
+	who = gdata_gd_who_new (GDATA_GD_WHO_EVENT_ORGANIZER, "John Smith‽", "john.smith@example.com");
+	gdata_calendar_event_add_person (event, who);
+	g_object_unref (who);
+	g_time_val_from_iso8601 ("2009-04-17T15:00:00.000Z", &start_time);
+	g_time_val_from_iso8601 ("2009-04-17T17:00:00.000Z", &end_time);
+	when = gdata_gd_when_new (start_time.tv_sec, end_time.tv_sec, FALSE);
+	gdata_calendar_event_add_time (event, when);
+	g_object_unref (when);
+
+	main_loop = g_main_loop_new (NULL, TRUE);
+
+	/* Insert the event */
+	gdata_calendar_service_insert_event_async (GDATA_CALENDAR_SERVICE (service), event, NULL, (GAsyncReadyCallback) test_insert_simple_async_cb,
+	                                           main_loop);
+	g_main_loop_run (main_loop);
+
+	g_main_loop_unref (main_loop);
+	g_object_unref (event);
+}
+
 static void
 test_xml_dates (void)
 {
@@ -299,7 +387,6 @@ test_xml_dates (void)
 	GList *times, *i;
 	GDataGDWhen *when;
 	gint64 _time;
-	gchar *xml;
 	GError *error = NULL;
 
 	event = GDATA_CALENDAR_EVENT (gdata_parsable_new_from_xml (GDATA_TYPE_CALENDAR_EVENT,
@@ -357,8 +444,7 @@ test_xml_dates (void)
 	g_assert (gdata_gd_when_get_reminders (when) == NULL);
 
 	/* Check the XML */
-	xml = gdata_parsable_get_xml (GDATA_PARSABLE (event));
-	g_assert_cmpstr (xml, ==,
+	gdata_test_assert_xml (event,
 			 "<?xml version='1.0' encoding='UTF-8'?>"
 			 "<entry xmlns='http://www.w3.org/2005/Atom' "
 			 	"xmlns:gd='http://schemas.google.com/g/2005' "
@@ -375,7 +461,6 @@ test_xml_dates (void)
 			 	"<gd:when startTime='2009-04-17T15:00:00Z'/>"
 			 	"<gd:when startTime='2009-04-27' endTime='2009-05-06'/>"
 			 "</entry>");
-	g_free (xml);
 
 	g_object_unref (event);
 }
@@ -444,6 +529,61 @@ test_xml_recurrence (void)
 	g_object_unref (event);
 }
 
+static void
+test_calendar_escaping (void)
+{
+	GDataCalendarCalendar *calendar;
+
+	calendar = gdata_calendar_calendar_new (NULL);
+	gdata_calendar_calendar_set_timezone (calendar, "<timezone>");
+
+	/* Check the outputted XML is escaped properly */
+	gdata_test_assert_xml (calendar,
+	                 "<?xml version='1.0' encoding='UTF-8'?>"
+	                 "<entry xmlns='http://www.w3.org/2005/Atom' xmlns:gd='http://schemas.google.com/g/2005' "
+	                        "xmlns:gCal='http://schemas.google.com/gCal/2005' xmlns:app='http://www.w3.org/2007/app'>"
+				"<title type='text'></title>"
+				"<category term='http://schemas.google.com/gCal/2005#calendarmeta' scheme='http://schemas.google.com/g/2005#kind'/>"
+				"<gCal:timezone value='&lt;timezone&gt;'/>"
+				"<gCal:hidden value='false'/>"
+				"<gCal:color value='#000000'/>"
+				"<gCal:selected value='false'/>"
+	                 "</entry>");
+	g_object_unref (calendar);
+}
+
+static void
+test_event_escaping (void)
+{
+	GDataCalendarEvent *event;
+
+	event = gdata_calendar_event_new (NULL);
+	gdata_calendar_event_set_status (event, "<status>");
+	gdata_calendar_event_set_visibility (event, "<visibility>");
+	gdata_calendar_event_set_transparency (event, "<transparency>");
+	gdata_calendar_event_set_uid (event, "<uid>");
+	gdata_calendar_event_set_recurrence (event, "<recurrence>");
+
+	/* Check the outputted XML is escaped properly */
+	gdata_test_assert_xml (event,
+	                 "<?xml version='1.0' encoding='UTF-8'?>"
+	                 "<entry xmlns='http://www.w3.org/2005/Atom' xmlns:gd='http://schemas.google.com/g/2005' "
+	                        "xmlns:gCal='http://schemas.google.com/gCal/2005' xmlns:app='http://www.w3.org/2007/app'>"
+				"<title type='text'></title>"
+				"<category term='http://schemas.google.com/g/2005#event' scheme='http://schemas.google.com/g/2005#kind'/>"
+				"<gd:eventStatus value='&lt;status&gt;'/>"
+				"<gd:visibility value='&lt;visibility&gt;'/>"
+				"<gd:transparency value='&lt;transparency&gt;'/>"
+				"<gCal:uid value='&lt;uid&gt;'/>"
+				"<gCal:guestsCanModify value='false'/>"
+				"<gCal:guestsCanInviteOthers value='false'/>"
+				"<gCal:guestsCanSeeGuests value='false'/>"
+				"<gCal:anyoneCanAddSelf value='false'/>"
+				"<gd:recurrence>&lt;recurrence&gt;</gd:recurrence>"
+	                 "</entry>");
+	g_object_unref (event);
+}
+
 static void
 test_query_uri (void)
 {
@@ -570,7 +710,6 @@ test_acls_insert_rule (gconstpointer service)
 	GDataCategory *category;
 	GDataLink *_link;
 	GList *categories;
-	gchar *xml;
 	gint64 edited;
 	GError *error = NULL;
 
@@ -587,8 +726,7 @@ test_acls_insert_rule (gconstpointer service)
 	g_assert_cmpstr (scope_value, ==, "darcy@gmail.com");
 
 	/* Check the XML */
-	xml = gdata_parsable_get_xml (GDATA_PARSABLE (rule));
-	g_assert_cmpstr (xml, ==,
+	gdata_test_assert_xml (rule,
 			 "<?xml version='1.0' encoding='UTF-8'?>"
 			 "<entry xmlns='http://www.w3.org/2005/Atom' "
 			 	"xmlns:gd='http://schemas.google.com/g/2005' "
@@ -598,7 +736,6 @@ test_acls_insert_rule (gconstpointer service)
 				"<gAcl:role value='http://schemas.google.com/gCal/2005#editor'/>"
 				"<gAcl:scope type='user' value='darcy@gmail.com'/>"
 			 "</entry>");
-	g_free (xml);
 
 	/* Insert the rule */
 	_link = gdata_entry_look_up_link (GDATA_ENTRY (calendar), GDATA_LINK_ACCESS_CONTROL_LIST);
@@ -769,7 +906,7 @@ test_batch (gconstpointer service)
 	gdata_entry_set_title (GDATA_ENTRY (event), "Fooish Bar");
 
 	gdata_test_batch_operation_insertion (operation, GDATA_ENTRY (event), &inserted_entry, NULL);
-	g_assert (gdata_batch_operation_run (operation, NULL, &error) == TRUE);
+	g_assert (gdata_test_batch_operation_run (operation, NULL, &error) == TRUE);
 	g_assert_no_error (error);
 
 	g_clear_error (&error);
@@ -786,7 +923,7 @@ test_batch (gconstpointer service)
 	                                           NULL);
 	g_assert_cmpuint (op_id, !=, op_id2);
 
-	g_assert (gdata_batch_operation_run (operation, NULL, &error) == TRUE);
+	g_assert (gdata_test_batch_operation_run (operation, NULL, &error) == TRUE);
 	g_assert_no_error (error);
 
 	g_clear_error (&error);
@@ -805,7 +942,7 @@ test_batch (gconstpointer service)
 	g_assert_cmpuint (op_id, !=, op_id3);
 	g_assert_cmpuint (op_id2, !=, op_id3);
 
-	g_assert (gdata_batch_operation_run (operation, NULL, &error) == TRUE);
+	g_assert (gdata_test_batch_operation_run (operation, NULL, &error) == TRUE);
 	g_assert_no_error (error);
 
 	g_assert_error (entry_error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR);
@@ -821,7 +958,7 @@ test_batch (gconstpointer service)
 	/* Turns out that we can't run this test, because the Calendar service sucks with ETags and doesn't error. */
 	/*operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "https://www.google.com/calendar/feeds/default/private/full/batch");
 	gdata_test_batch_operation_update (operation, inserted_entry2, NULL, &entry_error);
-	g_assert (gdata_batch_operation_run (operation, NULL, &error) == TRUE);
+	g_assert (gdata_test_batch_operation_run (operation, NULL, &error) == TRUE);
 	g_assert_no_error (error);
 
 	g_assert_error (entry_error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_CONFLICT);
@@ -834,7 +971,7 @@ test_batch (gconstpointer service)
 	/* Run a final batch operation to delete the second entry */
 	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "https://www.google.com/calendar/feeds/default/private/full/batch");
 	gdata_test_batch_operation_deletion (operation, inserted_entry3, NULL);
-	g_assert (gdata_batch_operation_run (operation, NULL, &error) == TRUE);
+	g_assert (gdata_test_batch_operation_run (operation, NULL, &error) == TRUE);
 	g_assert_no_error (error);
 
 	g_clear_error (&error);
@@ -870,7 +1007,10 @@ test_batch_async_cb (GDataBatchOperation *operation, GAsyncResult *async_result,
 {
 	GError *error = NULL;
 
-	g_assert (gdata_batch_operation_run_finish (operation, async_result, &error) == TRUE);
+	/* Clear all pending events (such as callbacks for the operations) */
+	while (g_main_context_iteration (NULL, FALSE) == TRUE);
+
+	g_assert (gdata_test_batch_operation_run_finish (operation, async_result, &error) == TRUE);
 	g_assert_no_error (error);
 	g_clear_error (&error);
 
@@ -903,7 +1043,10 @@ test_batch_async_cancellation_cb (GDataBatchOperation *operation, GAsyncResult *
 {
 	GError *error = NULL;
 
-	g_assert (gdata_batch_operation_run_finish (operation, async_result, &error) == FALSE);
+	/* Clear all pending events (such as callbacks for the operations) */
+	while (g_main_context_iteration (NULL, FALSE) == TRUE);
+
+	g_assert (gdata_test_batch_operation_run_finish (operation, async_result, &error) == FALSE);
 	g_assert_error (error, G_IO_ERROR, G_IO_ERROR_CANCELLED);
 	g_clear_error (&error);
 
@@ -917,11 +1060,12 @@ test_batch_async_cancellation (BatchAsyncData *data, gconstpointer service)
 	guint op_id;
 	GMainLoop *main_loop;
 	GCancellable *cancellable;
+	GError *error = NULL;
 
 	/* Run an async query operation on the event */
 	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "https://www.google.com/calendar/feeds/default/private/full/batch");
 	op_id = gdata_test_batch_operation_query (operation, gdata_entry_get_id (GDATA_ENTRY (data->new_event)), GDATA_TYPE_CALENDAR_EVENT,
-	                                          GDATA_ENTRY (data->new_event), NULL, NULL);
+	                                          GDATA_ENTRY (data->new_event), NULL, &error);
 
 	main_loop = g_main_loop_new (NULL, TRUE);
 	cancellable = g_cancellable_new ();
@@ -930,6 +1074,10 @@ test_batch_async_cancellation (BatchAsyncData *data, gconstpointer service)
 	g_cancellable_cancel (cancellable); /* this should cancel the operation before it even starts, as we haven't run the main loop yet */
 
 	g_main_loop_run (main_loop);
+
+	g_assert_error (error, G_IO_ERROR, G_IO_ERROR_CANCELLED);
+	g_clear_error (&error);
+
 	g_main_loop_unref (main_loop);
 	g_object_unref (cancellable);
 	g_object_unref (operation);
@@ -968,8 +1116,10 @@ main (int argc, char *argv[])
 		g_test_add_data_func ("/calendar/query/own_calendars", service, test_query_own_calendars);
 		g_test_add_data_func ("/calendar/query/own_calendars_async", service, test_query_own_calendars_async);
 		g_test_add_data_func ("/calendar/query/events", service, test_query_events);
+		g_test_add_data_func ("/calendar/query/events_async", service, test_query_events_async);
 
 		g_test_add_data_func ("/calendar/insert/simple", service, test_insert_simple);
+		g_test_add_data_func ("/calendar/insert/simple/async", service, test_insert_simple_async);
 
 		g_test_add_data_func ("/calendar/acls/get_rules", service, test_acls_get_rules);
 		g_test_add_data_func ("/calendar/acls/insert_rule", service, test_acls_insert_rule);
@@ -985,6 +1135,9 @@ main (int argc, char *argv[])
 	g_test_add_func ("/calendar/xml/dates", test_xml_dates);
 	g_test_add_func ("/calendar/xml/recurrence", test_xml_recurrence);
 
+	g_test_add_func ("/calendar/calendar/escaping", test_calendar_escaping);
+	g_test_add_func ("/calendar/event/escaping", test_event_escaping);
+
 	g_test_add_func ("/calendar/query/uri", test_query_uri);
 	g_test_add_func ("/calendar/query/etag", test_query_etag);
 
